@@ -19,19 +19,13 @@ Config::Config( Logger *_logger, QObject *parent )
 
     pPluginLoader = new PluginLoader( logger, this );
     pTagEngine = new TagEngine( this );
-    pConversionOptionsManager = new ConversionOptionsManager( pPluginLoader );
+    pConversionOptionsManager = new ConversionOptionsManager( pPluginLoader, this );
 }
 
 Config::~Config()
 {
     save();
-
-    qDeleteAll( data.profiles.values() );
-    data.profiles.clear();
-
-    delete pPluginLoader;
-    delete pTagEngine;
-    delete pConversionOptionsManager;
+    qDeleteAll(data.profiles);
 }
 
 void Config::load()
@@ -60,6 +54,7 @@ void Config::load()
     data.general.lastNormalOutputDirectoryPaths = group.readEntry( "lastNormalOutputDirectoryPaths", QStringList() );
     data.general.waitForAlbumGain = group.readEntry( "waitForAlbumGain", true );
     data.general.useVFATNames = group.readEntry( "useVFATNames", false );
+    data.general.copyIfSameCodec = group.readEntry( "copyIfSameCodec", false );
     data.general.writeLogFiles = group.readEntry( "writeLogFiles", false );
     data.general.conflictHandling = (Config::Data::General::ConflictHandling)group.readEntry( "conflictHandling", 0 );
 //     data.general.priority = group.readEntry( "priority", 10 );
@@ -111,6 +106,7 @@ void Config::load()
     }
     data.advanced.maxSizeForSharedMemoryTempFiles = group.readEntry( "maxSizeForSharedMemoryTempFiles", data.advanced.sharedMemorySize / 4 );
     data.advanced.usePipes = group.readEntry( "usePipes", false );
+    data.advanced.ejectCdAfterRip = group.readEntry( "ejectCdAfterRip", true );
 
     group = conf->group( "CoverArt" );
     data.coverArt.writeCovers = group.readEntry( "writeCovers", 1 );
@@ -118,9 +114,8 @@ void Config::load()
     data.coverArt.writeCoverDefaultName = group.readEntry( "writeCoverDefaultName", i18nc("cover file name","cover") );
 
     group = conf->group( "Backends" );
-    data.backends.rippers = group.readEntry( "rippers", QStringList() );
     formats = group.readEntry( "formats", QStringList() );
-    foreach( const QString format, formats )
+    foreach( const QString& format, formats )
     {
         CodecData codecData;
         codecData.codecName = format;
@@ -135,7 +130,6 @@ void Config::load()
     pPluginLoader->load();
 
     QString pluginName;
-    bool found;
     QStringList enabledPlugins;
     QStringList newPlugins;
     int codecIndex;
@@ -144,43 +138,7 @@ void Config::load()
 
     // build default backend priority list
 
-    // rippers
-    enabledPlugins.clear();
-    newPlugins.clear();
-    // register existing enabled plugins as such and list new enabled plugins
-    foreach( RipperPlugin *plugin, pPluginLoader->getAllRipperPlugins() )
-    {
-        pluginName = plugin->name();
-        foreach( const ConversionPipeTrunk trunk, plugin->codecTable() )
-        {
-            if( trunk.enabled )
-            {
-                enabledPlugins += pluginName;
-                if( !data.backends.rippers.contains(pluginName) && newPlugins.filter(QRegExp("[0-9]{8,8}"+pluginName)).count()==0 )
-                {
-                    newPlugins += QString::number(trunk.rating).rightJustified(8,'0') + pluginName;
-                    break;
-                }
-            }
-        }
-    }
-    // remove plugins from the ripper list if they aren't enabled any more
-    for( int i=0; i<data.backends.rippers.count(); i++ )
-    {
-        if( !enabledPlugins.contains(data.backends.rippers.at(i)) )
-        {
-            data.backends.rippers.removeAt(i);
-            i--;
-        }
-    }
-    // sort new enabled plugins and append them to the ripper list
-    newPlugins.sort();
-    for( int i=0; i<newPlugins.count(); i++ )
-    {
-        data.backends.rippers += newPlugins.at(i).right(newPlugins.at(i).length()-8);
-    }
-
-    foreach( const QString format, formats )
+    foreach( const QString& format, formats )
     {
         if( format == "wav" )
             continue;
@@ -208,7 +166,7 @@ void Config::load()
         enabledPlugins.clear();
         newPlugins.clear();
         // register existing enabled plugins as such and list new enabled plugins
-        foreach( const ConversionPipeTrunk trunk, pPluginLoader->conversionFilterPipeTrunks )
+        foreach( const ConversionPipeTrunk& trunk, pPluginLoader->conversionFilterPipeTrunks )
         {
             if( trunk.codecTo == format && trunk.enabled )
             {
@@ -240,7 +198,7 @@ void Config::load()
         enabledPlugins.clear();
         newPlugins.clear();
         // register existing enabled plugins as such and list new enabled plugins
-        foreach( const ConversionPipeTrunk trunk, pPluginLoader->conversionFilterPipeTrunks )
+        foreach( const ConversionPipeTrunk& trunk, pPluginLoader->conversionFilterPipeTrunks )
         {
             if( trunk.codecFrom == format && trunk.enabled )
             {
@@ -277,7 +235,7 @@ void Config::load()
         }
         newPlugins.clear();
         // register existing enabled plugins as such and list new enabled plugins
-        foreach( const ReplayGainPipe pipe, pPluginLoader->replaygainPipes )
+        foreach( const ReplayGainPipe& pipe, pPluginLoader->replaygainPipes )
         {
             if( pipe.codecName == format && pipe.enabled )
             {
@@ -318,7 +276,7 @@ void Config::load()
     foreach( FilterPlugin *plugin, pPluginLoader->getAllFilterPlugins() )
     {
         pluginName = plugin->name();
-        foreach( const ConversionPipeTrunk trunk, plugin->codecTable() )
+        foreach( const ConversionPipeTrunk& trunk, plugin->codecTable() )
         {
             if( trunk.enabled && trunk.codecFrom == "wav" && trunk.codecTo == "wav" )
             {
@@ -375,14 +333,38 @@ void Config::load()
                 QDomNodeList conversionOptionsElements = root.elementsByTagName("conversionOptions");
                 for( int i=0; i<conversionOptionsElements.count(); i++ )
                 {
-                    ConversionOptions *conversionOptions = 0;
-                    QList<QDomElement> filterOptionsElements;
                     const QString profileName = conversionOptionsElements.at(i).toElement().attribute("profileName");
                     const QString pluginName = conversionOptionsElements.at(i).toElement().attribute("pluginName");
-                    CodecPlugin *plugin = (CodecPlugin*)pPluginLoader->backendPluginByName( pluginName );
+                    CodecPlugin *plugin = qobject_cast<CodecPlugin*>(pPluginLoader->backendPluginByName( pluginName ));
                     if( plugin )
                     {
-                        conversionOptions = plugin->conversionOptionsFromXml( conversionOptionsElements.at(i).toElement(), &filterOptionsElements );
+                        QList<QDomElement> filterOptionsElements;
+                        ConversionOptions *conversionOptions = plugin->conversionOptionsFromXml( conversionOptionsElements.at(i).toElement(), &filterOptionsElements );
+                        if( conversionOptions )
+                        {
+                            foreach( const QDomElement& filterOptionsElement, filterOptionsElements )
+                            {
+                                FilterOptions *filterOptions = 0;
+                                const QString filterPluginName = filterOptionsElement.attribute("pluginName");
+                                FilterPlugin *filterPlugin = qobject_cast<FilterPlugin*>(pPluginLoader->backendPluginByName( filterPluginName ));
+                                if( filterPlugin )
+                                {
+                                    filterOptions = filterPlugin->filterOptionsFromXml( filterOptionsElement );
+                                }
+                                else
+                                {
+                                    logger->log( 1000, "\tcannot load filter for profile: " + profileName );
+                                    continue;
+                                }
+                                conversionOptions->filterOptions.append( filterOptions );
+                            }
+
+                            pConversionOptionsManager->addConversionOptions( conversionOptions );
+
+                            data.profiles[profileName] = conversionOptions->copy();
+                            if( profileName != "soundkonverter_last_used" )
+                                logger->log( 1000, "\tname: " + profileName + ", plugin: " + pluginName );
+                        }
                     }
                     else
                     {
@@ -390,31 +372,6 @@ void Config::load()
                         logger->log( 1000, "\t\tcannot be loaded beacause the plugin cannot be found" );
                         continue;
                     }
-                    if( conversionOptions )
-                    {
-                        foreach( QDomElement filterOptionsElement, filterOptionsElements )
-                        {
-                            FilterOptions *filterOptions = 0;
-                            const QString filterPluginName = filterOptionsElement.attribute("pluginName");
-                            FilterPlugin *filterPlugin = (FilterPlugin*)pPluginLoader->backendPluginByName( filterPluginName );
-                            if( filterPlugin )
-                            {
-                                filterOptions = filterPlugin->filterOptionsFromXml( filterOptionsElement );
-                            }
-                            else
-                            {
-                                logger->log( 1000, "\tcannot load filter for profile: " + profileName );
-                                continue;
-                            }
-                            conversionOptions->filterOptions.append( filterOptions );
-                        }
-                    }
-                    if( conversionOptions )
-                    {
-                        data.profiles[profileName] = conversionOptions;
-                        if( profileName != "soundkonverter_last_used" )
-                            logger->log( 1000, "\tname: " + profileName + ", plugin: " + pluginName );
-                    }
                 }
             }
         }
@@ -460,7 +417,7 @@ void Config::load()
         }
         else
         {
-            ConversionOptions *conversionOptions = data.profiles.value( profile );
+            const ConversionOptions *conversionOptions = data.profiles.value( profile );
             if( conversionOptions )
                 sFormat += conversionOptions->codecName;
         }
@@ -497,6 +454,8 @@ void Config::load()
         data.backendOptimizationsIgnoreList.optimizationList.append(optimization);
     }
 
+    writeServiceMenu();
+
     logger->log( 1000, QString("loading of the configuration took %1 ms").arg(time.elapsed()) );
 }
 
@@ -523,6 +482,7 @@ void Config::save()
     group.writeEntry( "lastNormalOutputDirectoryPaths", data.general.lastNormalOutputDirectoryPaths );
     group.writeEntry( "waitForAlbumGain", data.general.waitForAlbumGain );
     group.writeEntry( "useVFATNames", data.general.useVFATNames );
+    group.writeEntry( "copyIfSameCodec", data.general.copyIfSameCodec );
     group.writeEntry( "writeLogFiles", data.general.writeLogFiles );
     group.writeEntry( "conflictHandling", (int)data.general.conflictHandling );
 //     group.writeEntry( "priority", data.general.priority );
@@ -543,6 +503,7 @@ void Config::save()
     group.writeEntry( "useSharedMemoryForTempFiles", data.advanced.useSharedMemoryForTempFiles );
     group.writeEntry( "maxSizeForSharedMemoryTempFiles", data.advanced.maxSizeForSharedMemoryTempFiles );
     group.writeEntry( "usePipes", data.advanced.usePipes );
+    group.writeEntry( "ejectCdAfterRip", data.advanced.ejectCdAfterRip );
 
     group = conf->group( "CoverArt" );
     group.writeEntry( "writeCovers", data.coverArt.writeCovers );
@@ -550,9 +511,9 @@ void Config::save()
     group.writeEntry( "writeCoverDefaultName", data.coverArt.writeCoverDefaultName );
 
     group = conf->group( "Backends" );
-    group.writeEntry( "rippers", data.backends.rippers );
+    group.deleteEntry( "rippers" );
     QStringList formats;
-    foreach( const CodecData codec, data.backends.codecs )
+    foreach( const CodecData& codec, data.backends.codecs )
     {
         const QString format = codec.codecName;
         group.writeEntry( format + "_encoders", codec.encoders );
@@ -604,7 +565,7 @@ void Config::writeServiceMenu()
     const QStringList convertFormats = pPluginLoader->formatList( PluginLoader::Decode, PluginLoader::CompressionType(PluginLoader::InferiorQuality|PluginLoader::Lossy|PluginLoader::Lossless|PluginLoader::Hybrid) );
 
     mimeTypes.clear();
-    foreach( const QString format, convertFormats )
+    foreach( const QString& format, convertFormats )
     {
         mimeTypes += pPluginLoader->codecMimeTypes( format );
     }
@@ -620,7 +581,7 @@ void Config::writeServiceMenu()
     content += "Exec=soundkonverter %F\n";
 
     const QString convertActionFileName = KStandardDirs::locateLocal( "services", "ServiceMenus/convert_with_soundkonverter.desktop" );
-    if( ( data.general.actionMenuConvertMimeTypes != mimeTypes || QFile::exists(convertActionFileName) ) && mimeTypes.count() > 0 )
+    if( ( data.general.actionMenuConvertMimeTypes != mimeTypes || !QFile::exists(convertActionFileName) ) && mimeTypes.count() > 0 )
     {
         QFile convertActionFile( convertActionFileName );
         if( convertActionFile.open( QIODevice::WriteOnly | QIODevice::Text ) )
@@ -640,7 +601,7 @@ void Config::writeServiceMenu()
     const QStringList replaygainFormats = pPluginLoader->formatList( PluginLoader::ReplayGain, PluginLoader::CompressionType(PluginLoader::InferiorQuality|PluginLoader::Lossy|PluginLoader::Lossless|PluginLoader::Hybrid) );
 
     mimeTypes.clear();
-    foreach( const QString format, replaygainFormats )
+    foreach( const QString& format, replaygainFormats )
     {
         mimeTypes += pPluginLoader->codecMimeTypes( format );
     }
@@ -656,7 +617,7 @@ void Config::writeServiceMenu()
     content += "Exec=soundkonverter --replaygain %F\n";
 
     const QString replaygainActionFileName = KStandardDirs::locateLocal( "services", "ServiceMenus/add_replaygain_with_soundkonverter.desktop" );
-    if( ( data.general.actionMenuReplayGainMimeTypes != mimeTypes || QFile::exists(replaygainActionFileName) ) && mimeTypes.count() > 0 )
+    if( ( data.general.actionMenuReplayGainMimeTypes != mimeTypes || !QFile::exists(replaygainActionFileName) ) && mimeTypes.count() > 0 )
     {
         QFile replaygainActionFile( replaygainActionFileName );
         if( replaygainActionFile.open( QIODevice::WriteOnly | QIODevice::Text ) )
@@ -673,15 +634,17 @@ QStringList Config::customProfiles()
 {
     QStringList profiles;
 
-    foreach( const QString profileName, data.profiles.keys() )
+    foreach( const QString& profileName, data.profiles.keys() )
     {
         if( profileName == "soundkonverter_last_used" )
             continue;
 
-        QList<CodecPlugin*> plugins = pPluginLoader->encodersForCodec( data.profiles.value(profileName)->codecName );
-        foreach( CodecPlugin *plugin, plugins )
+        const ConversionOptions* profileConversionOptions = data.profiles.value(profileName);
+
+        QList<CodecPlugin*> plugins = pPluginLoader->encodersForCodec( profileConversionOptions->codecName );
+        foreach( const CodecPlugin *plugin, plugins )
         {
-            if( plugin->name() == data.profiles.value(profileName)->pluginName )
+            if( plugin->name() == profileConversionOptions->pluginName )
             {
                 profiles.append( profileName );
                 break;
@@ -708,7 +671,7 @@ QList<CodecOptimizations::Optimization> Config::getOptimizations( bool includeIg
     bool ignore;
 
     const QStringList formats = pPluginLoader->formatList( PluginLoader::Possibilities(PluginLoader::Encode|PluginLoader::Decode|PluginLoader::ReplayGain), PluginLoader::CompressionType(PluginLoader::InferiorQuality|PluginLoader::Lossy|PluginLoader::Lossless|PluginLoader::Hybrid) );
-    foreach( const QString format, formats )
+    foreach( const QString& format, formats )
     {
         if( format == "wav" )
             continue;
