@@ -125,7 +125,7 @@ BackendPlugin::FormatInfo BackendPlugin::formatInfo( const QString& codecName )
     }
     else if( codecName == "ac3" )
     {
-        info.description = i18n("AC3 or AC-3 is part of the Dolby Digital format family.<br>It was designed to be used for movies (e.g. DVDs) and supports sourround sound.<br><br><a href=\"http://en.wikipedia.org/wiki/Dolby_Digital\">http://en.wikipedia.org/wiki/Dolby Digital</a>");
+        info.description = i18n("AC3 or AC-3 is part of the Dolby Digital format family.<br>It was designed to be used for movies (e.g. DVDs) and supports surround sound.<br><br><a href=\"http://en.wikipedia.org/wiki/Dolby_Digital\">http://en.wikipedia.org/wiki/Dolby Digital</a>");
         info.mimeTypes.append( "audio/ac3" );
         info.extensions.append( "ac3" );
     }
@@ -290,6 +290,11 @@ BackendPlugin::FormatInfo BackendPlugin::formatInfo( const QString& codecName )
         info.mimeTypes.append( "video/x-matroska" );
         info.extensions.append( "mkv" );
     }
+    else if( codecName == "webm" )
+    {
+        info.mimeTypes.append( "video/webm" );
+        info.extensions.append( "webm" );
+    }
     else if( codecName == "ogv" )
     {
         info.mimeTypes.append( "video/ogg" );
@@ -514,7 +519,7 @@ void BackendPlugin::processExit( int exitCode, QProcess::ExitStatus exitStatus )
     }
 }
 
-QString BackendPlugin::standardMessage( const QString& type, const QStringList& arguments )
+QString BackendPlugin::standardMessage( const QString& type, const QStringList& arguments ) const
 {
     if( type == "encode_codec,backend" )
     {
@@ -583,17 +588,17 @@ QString BackendPlugin::standardMessage( const QString& type, const QStringList&
     return QString();
 }
 
-QString BackendPlugin::standardMessage( const QString& type )
+QString BackendPlugin::standardMessage( const QString& type ) const
 {
     return standardMessage( type, QStringList() );
 }
 
-QString BackendPlugin::standardMessage( const QString& type, const QString& argument1 )
+QString BackendPlugin::standardMessage( const QString& type, const QString& argument1 ) const
 {
     return standardMessage( type, QStringList(argument1) );
 }
 
-QString BackendPlugin::standardMessage(const QString& type, const QString& argument1, const QString& argument2)
+QString BackendPlugin::standardMessage(const QString& type, const QString& argument1, const QString& argument2) const
 {
     QStringList arguments;
     arguments.append( argument1 );
@@ -601,7 +606,7 @@ QString BackendPlugin::standardMessage(const QString& type, const QString& argum
     return standardMessage( type, arguments );
 }
 
-QString BackendPlugin::standardMessage(const QString& type, const QString& argument1, const QString& argument2, const QString& argument3)
+QString BackendPlugin::standardMessage(const QString& type, const QString& argument1, const QString& argument2, const QString& argument3) const
 {
     QStringList arguments;
     arguments.append( argument1 );
