@@ -6,6 +6,7 @@
 #include "outputdirectory.h"
 #include "global.h"
 
+#include <QApplication>
 #include <QLayout>
 #include <QBoxLayout>
 #include <QLabel>
@@ -32,6 +33,8 @@ OptionsDetailed::OptionsDetailed( Config* _config, QWidget* parent )
     : QWidget( parent ),
     config( _config )
 {
+    const int fontHeight = QFontMetrics(QApplication::font()).boundingRect("M").size().height();
+
     int gridRow = 0;
     grid = new QGridLayout( this );
 
@@ -66,7 +69,7 @@ OptionsDetailed::OptionsDetailed( Config* _config, QWidget* parent )
     QFrame *lineFrame = new QFrame( this );
     lineFrame->setFrameShape( QFrame::HLine );
     lineFrame->setFrameShadow( QFrame::Sunken );
-    lineFrame->setFixedHeight( 10 );
+    lineFrame->setFixedHeight( fontHeight );
     grid->addWidget( lineFrame, 1, 0 );
 
     // prepare the plugin widget
@@ -79,11 +82,11 @@ OptionsDetailed::OptionsDetailed( Config* _config, QWidget* parent )
     lineFrame = new QFrame( this );
     lineFrame->setFrameShape( QFrame::HLine );
     lineFrame->setFrameShadow( QFrame::Sunken );
-    lineFrame->setFixedHeight( 10 );
+    lineFrame->setFixedHeight( fontHeight );
     grid->addWidget( lineFrame, gridRow++, 0 );
 
     int filterCount = 0;
-    foreach( QString pluginName, config->data.backends.enabledFilters )
+    foreach( const QString& pluginName, config->data.backends.enabledFilters )
     {
         FilterPlugin *plugin = qobject_cast<FilterPlugin*>(config->pluginLoader()->backendPluginByName(pluginName));
         if( !plugin )
@@ -98,7 +101,6 @@ OptionsDetailed::OptionsDetailed( Config* _config, QWidget* parent )
         grid->addWidget( widget, gridRow++, 0 );
         widget->show();
         filterCount++;
-        break; // show only one widget at the moment
     }
     if( filterCount > 0 )
     {
@@ -106,7 +108,7 @@ OptionsDetailed::OptionsDetailed( Config* _config, QWidget* parent )
         lineFrame = new QFrame( this );
         lineFrame->setFrameShape( QFrame::HLine );
         lineFrame->setFrameShadow( QFrame::Sunken );
-        lineFrame->setFixedHeight( 10 );
+        lineFrame->setFixedHeight( fontHeight );
         grid->addWidget( lineFrame, gridRow++, 0 );
     }
 
@@ -199,15 +201,17 @@ CodecPlugin *OptionsDetailed::getCurrentPlugin()
 
 void OptionsDetailed::updateProfiles()
 {
+    if( pProfileLoad->menu() )
+        pProfileLoad->menu()->deleteLater();
+
     QMenu *menu = new QMenu( this );
-    const QStringList profiles = config->customProfiles();
-    for( int i=0; i<profiles.count(); i++ )
+    foreach( const QString& profile, config->customProfiles() )
     {
-        menu->addAction( profiles.at(i), this, SLOT(loadCustomProfileButtonClicked()) );
+        menu->addAction( profile, this, SLOT(loadCustomProfileButtonClicked()) );
     }
 
     pProfileLoad->setMenu( menu );
-    pProfileLoad->setShown( profiles.count() > 0 );
+    pProfileLoad->setShown( config->customProfiles().count() > 0 );
 }
 
 void OptionsDetailed::formatChanged( const QString& format )
@@ -268,25 +272,29 @@ void OptionsDetailed::formatChanged( const QString& format )
 
 void OptionsDetailed::encoderChanged( const QString& encoder )
 {
-    CodecPlugin *plugin = (CodecPlugin*)config->pluginLoader()->backendPluginByName( encoder );
+    CodecPlugin *plugin = qobject_cast<CodecPlugin*>(config->pluginLoader()->backendPluginByName( encoder ));
     if( !plugin )
     {
 //         TODO leads to crashes
 //         KMessageBox::error( this, i18n("Sorry, this shouldn't happen.\n\nPlease report this bug and attach the following error message:\n\nOptionsDetailed::encoderChanged; PluginLoader::codecPluginByName returned 0 for encoder: '%1'").arg(encoder), i18n("Internal error") );
         return;
     }
-    currentPlugin = plugin;
     if( wPlugin )
     {
         grid->removeWidget( wPlugin );
         disconnect( wPlugin, SIGNAL(optionsChanged()), 0, 0 );
-        wPlugin = plugin->deleteCodecWidget( wPlugin );
+        wPlugin = currentPlugin->deleteCodecWidget( wPlugin );
     }
+    currentPlugin = plugin;
     wPlugin = plugin->newCodecWidget();
     if( wPlugin )
     {
         connect( wPlugin, SIGNAL(optionsChanged()), this, SLOT(somethingChanged()) );
         qobject_cast<CodecWidget*>(wPlugin)->setCurrentFormat( cFormat->currentText() );
+        if( plugin->lastConversionOptions() )
+        {
+            wPlugin->setCurrentConversionOptions( plugin->lastConversionOptions() );
+        }
         grid->addWidget( wPlugin, 2, 0 );
     }
 
@@ -322,7 +330,7 @@ void OptionsDetailed::somethingChanged()
 
 void OptionsDetailed::configurePlugin()
 {
-    CodecPlugin *plugin = (CodecPlugin*)config->pluginLoader()->backendPluginByName( cPlugin->currentText() );
+    CodecPlugin *plugin = qobject_cast<CodecPlugin*>(config->pluginLoader()->backendPluginByName( cPlugin->currentText() ));
 
     if( plugin )
     {
@@ -332,23 +340,21 @@ void OptionsDetailed::configurePlugin()
 
 ConversionOptions *OptionsDetailed::currentConversionOptions( bool saveLastUsed )
 {
-    ConversionOptions *options = 0;
-
     if( wPlugin && currentPlugin )
     {
-        options = wPlugin->currentConversionOptions();
-        if( options )
+        ConversionOptions *conversionOptions = wPlugin->currentConversionOptions();
+        if( conversionOptions )
         {
-            options->codecName = cFormat->currentText();
-            if( options->codecName != "wav" )
-                options->pluginName = currentPlugin->name();
+            conversionOptions->codecName = cFormat->currentText();
+            if( conversionOptions->codecName != "wav" )
+                conversionOptions->pluginName = currentPlugin->name();
             else
-                options->pluginName = "";
-            options->profile = wPlugin->currentProfile();
-            options->outputDirectoryMode = outputDirectory->mode();
-            options->outputDirectory = outputDirectory->directory();
-            options->outputFilesystem = outputDirectory->filesystem();
-            options->replaygain = cReplayGain->isChecked();
+                conversionOptions->pluginName = "";
+            conversionOptions->profile = wPlugin->currentProfile();
+            conversionOptions->outputDirectoryMode = outputDirectory->mode();
+            conversionOptions->outputDirectory = outputDirectory->directory();
+            conversionOptions->outputFilesystem = outputDirectory->filesystem();
+            conversionOptions->replaygain = cReplayGain->isEnabled() && cReplayGain->isChecked();
 
             for( int i=0; i<wFilter.size(); i++ )
             {
@@ -360,7 +366,7 @@ ConversionOptions *OptionsDetailed::currentConversionOptions( bool saveLastUsed
                     if( filterOptions )
                     {
                         filterOptions->pluginName = plugin->name();
-                        options->filterOptions.append( filterOptions );
+                        conversionOptions->filterOptions.append( filterOptions );
                     }
                 }
             }
@@ -371,39 +377,41 @@ ConversionOptions *OptionsDetailed::currentConversionOptions( bool saveLastUsed
                 saveCustomProfile( true );
                 config->data.general.lastFormat = cFormat->currentText();
             }
+
+            return conversionOptions;
         }
     }
 
-    return options;
+    return 0;
 }
 
-bool OptionsDetailed::setCurrentConversionOptions( ConversionOptions *options )
+bool OptionsDetailed::setCurrentConversionOptions( const ConversionOptions *conversionOptions )
 {
-    if( !options )
+    if( !conversionOptions )
         return false;
 
-    cFormat->setCurrentIndex( cFormat->findText(options->codecName) );
+    cFormat->setCurrentIndex( cFormat->findText(conversionOptions->codecName) );
     formatChanged( cFormat->currentText() );
-    if( options->codecName != "wav" )
+    if( conversionOptions->codecName != "wav" )
     {
-        cPlugin->setCurrentIndex( cPlugin->findText(options->pluginName) );
+        cPlugin->setCurrentIndex( cPlugin->findText(conversionOptions->pluginName) );
         encoderChanged( cPlugin->currentText() );
     }
-    outputDirectory->setMode( (OutputDirectory::Mode)options->outputDirectoryMode );
-    outputDirectory->setDirectory( options->outputDirectory );
-    cReplayGain->setChecked( options->replaygain );
+    outputDirectory->setMode( (OutputDirectory::Mode)conversionOptions->outputDirectoryMode );
+    outputDirectory->setDirectory( conversionOptions->outputDirectory );
+    cReplayGain->setChecked( conversionOptions->replaygain );
 
     bool succeeded = true;
 
-    if( options->codecName == "wav" )
+    if( conversionOptions->codecName == "wav" )
         succeeded = true;
     else if( wPlugin )
-        succeeded = wPlugin->setCurrentConversionOptions( options );
+        succeeded = wPlugin->setCurrentConversionOptions( conversionOptions );
     else
         succeeded = false;
 
     QStringList usedFilter;
-    foreach( FilterOptions *filterOptions, options->filterOptions )
+    foreach( const FilterOptions *filterOptions, conversionOptions->filterOptions )
     {
         bool filterSucceeded = false;
         for( int i=0; i<wFilter.size(); i++ )
@@ -428,7 +436,6 @@ bool OptionsDetailed::setCurrentConversionOptions( ConversionOptions *options )
         if( widget && plugin && !usedFilter.contains(plugin->name()) )
         {
             widget->setCurrentFilterOptions( 0 );
-            break;
         }
     }
 
@@ -580,7 +587,7 @@ bool OptionsDetailed::setCurrentProfile( const QString& profile )
 {
     if( config->data.profiles.keys().contains(profile) )
     {
-        ConversionOptions *conversionOptions = config->data.profiles.value( profile );
+        const ConversionOptions *conversionOptions = config->data.profiles.value( profile );
         if( conversionOptions )
             return setCurrentConversionOptions( conversionOptions );
     }
