@@ -4,16 +4,16 @@
 #include "normalizefilterwidget.h"
 #include "normalizefilteroptions.h"
 
+#include <QLayout>
+#include <QHBoxLayout>
 #include <KLocale>
 #include <QCheckBox>
-#include <QLayout>
 
 NormalizeFilterWidget::NormalizeFilterWidget()
     : FilterWidget()
 {
     QGridLayout *grid = new QGridLayout( this );
     grid->setContentsMargins( 0, 0, 0, 0 );
-    grid->setSpacing( 6 );
 
     // set up encoding options selection
 
@@ -48,12 +48,19 @@ FilterOptions* NormalizeFilterWidget::currentFilterOptions()
     }
 }
 
-bool NormalizeFilterWidget::setCurrentFilterOptions( FilterOptions *_options )
+bool NormalizeFilterWidget::setCurrentFilterOptions( const FilterOptions *_options )
 {
-    if( !_options || _options->pluginName != global_plugin_name )
+    if( !_options )
+    {
+        cNormalize->setChecked( false );
+
+        return true;
+    }
+
+    if( _options->pluginName != global_plugin_name )
         return false;
 
-    NormalizeFilterOptions *options = dynamic_cast<NormalizeFilterOptions*>(_options);
+    const NormalizeFilterOptions *options = dynamic_cast<const NormalizeFilterOptions*>(_options);
     cNormalize->setChecked( options->data.normalize );
 
     return true;
