@@ -20,6 +20,8 @@ soundkonverter_filter_sox::soundkonverter_filter_sox( QObject *parent, const QSt
 {
     Q_UNUSED(args)
 
+    configDialogSamplingRateQualityComboBox = 0;
+
     binaries["sox"] = "";
 
     KSharedConfig::Ptr conf = KGlobal::config();
@@ -118,7 +120,7 @@ soundkonverter_filter_sox::soundkonverter_filter_sox( QObject *parent, const QSt
 soundkonverter_filter_sox::~soundkonverter_filter_sox()
 {}
 
-QString soundkonverter_filter_sox::name()
+QString soundkonverter_filter_sox::name() const
 {
     return global_plugin_name;
 }
@@ -166,13 +168,13 @@ QList<ConversionPipeTrunk> soundkonverter_filter_sox::codecTable()
         allCodecs += codecList.at(i).codecName;
     }
 
-    foreach( const QString fromCodec, allCodecs )
+    foreach( const QString& fromCodec, allCodecs )
     {
-        foreach( const QString toCodec, allCodecs )
+        foreach( const QString& toCodec, allCodecs )
         {
             bool codecEnabled = false;
             QStringList soxProblemInfo;
-            foreach( const SoxCodecData data, codecList )
+            foreach( const SoxCodecData& data, codecList )
             {
                 if( data.codecName == toCodec )
                 {
@@ -300,8 +302,6 @@ FilterWidget *soundkonverter_filter_sox::newFilterWidget()
     if( lastUsedFilterOptions )
     {
         widget->setCurrentFilterOptions( lastUsedFilterOptions );
-        delete lastUsedFilterOptions;
-        lastUsedFilterOptions = 0;
     }
     return qobject_cast<FilterWidget*>(widget);
 }
@@ -309,16 +309,10 @@ FilterWidget *soundkonverter_filter_sox::newFilterWidget()
 CodecWidget *soundkonverter_filter_sox::newCodecWidget()
 {
     SoxCodecWidget *widget = new SoxCodecWidget();
-    if( lastUsedConversionOptions )
-    {
-        widget->setCurrentConversionOptions( lastUsedConversionOptions );
-        delete lastUsedConversionOptions;
-        lastUsedConversionOptions = 0;
-    }
     return qobject_cast<CodecWidget*>(widget);
 }
 
-unsigned int soundkonverter_filter_sox::convert( const KUrl& inputFile, const KUrl& outputFile, const QString& inputCodec, const QString& outputCodec, ConversionOptions *_conversionOptions, TagData *tags, bool replayGain )
+int soundkonverter_filter_sox::convert( const KUrl& inputFile, const KUrl& outputFile, const QString& inputCodec, const QString& outputCodec, const ConversionOptions *_conversionOptions, TagData *tags, bool replayGain )
 {
     QStringList command = convertCommand( inputFile, outputFile, inputCodec, outputCodec, _conversionOptions, tags, replayGain );
     if( command.isEmpty() )
@@ -341,7 +335,7 @@ unsigned int soundkonverter_filter_sox::convert( const KUrl& inputFile, const KU
     return newItem->id;
 }
 
-QStringList soundkonverter_filter_sox::convertCommand( const KUrl& inputFile, const KUrl& outputFile, const QString& inputCodec, const QString& outputCodec, ConversionOptions *_conversionOptions, TagData *tags, bool replayGain )
+QStringList soundkonverter_filter_sox::convertCommand( const KUrl& inputFile, const KUrl& outputFile, const QString& inputCodec, const QString& outputCodec, const ConversionOptions *_conversionOptions, TagData *tags, bool replayGain )
 {
     Q_UNUSED( tags );
     Q_UNUSED( replayGain );
@@ -349,15 +343,15 @@ QStringList soundkonverter_filter_sox::convertCommand( const KUrl& inputFile, co
     if( !_conversionOptions )
         return QStringList();
 
-    ConversionOptions *conversionOptions = _conversionOptions;
+    const ConversionOptions *conversionOptions = _conversionOptions;
 
     QStringList command;
 
-    SoxFilterOptions *filterOptions = 0;
-    foreach( FilterOptions *_filterOptions,conversionOptions->filterOptions )
+    const SoxFilterOptions *filterOptions = 0;
+    foreach( const FilterOptions *_filterOptions, conversionOptions->filterOptions )
     {
         if( _filterOptions->pluginName == global_plugin_name )
-            filterOptions = dynamic_cast<SoxFilterOptions*>(_filterOptions);
+            filterOptions = dynamic_cast<const SoxFilterOptions*>(_filterOptions);
     }
 
     command += binaries["sox"];
@@ -377,6 +371,11 @@ QStringList soundkonverter_filter_sox::convertCommand( const KUrl& inputFile, co
         command += "--bits";
         command += QString::number(filterOptions->data.sampleSize);
     }
+    if( filterOptions && filterOptions->data.channels )
+    {
+        command += "--channels";
+        command += QString::number(filterOptions->data.channels);
+    }
     if( outputCodec == "flac" && ( conversionOptions->pluginName == global_plugin_name || conversionOptions->pluginName == "FLAC" ) )
     {
         command += "--compression";
@@ -440,6 +439,17 @@ QStringList soundkonverter_filter_sox::convertCommand( const KUrl& inputFile, co
             command += "-v";
         command += QString::number(filterOptions->data.sampleRate);
     }
+    if( filterOptions )
+    {
+        foreach( const SoxFilterOptions::EffectData& effectData, filterOptions->data.effects )
+        {
+            if( effectData.effectName == "norm" || effectData.effectName == "bass" || effectData.effectName == "treble" )
+            {
+                command += effectData.effectName;
+                command += QString::number(effectData.data.at(0).toDouble());
+            }
+        }
+    }
 
     return command;
 }
@@ -469,7 +479,7 @@ FilterOptions *soundkonverter_filter_sox::filterOptionsFromXml( QDomElement filt
 
 QString soundkonverter_filter_sox::soxCodecName( const QString& codecName )
 {
-    foreach( SoxCodecData data, codecList )
+    foreach( const SoxCodecData& data, codecList )
     {
         if( data.codecName == codecName )
             return data.soxCodecName;
