@@ -119,25 +119,32 @@ void ReplayGainFileList::dropEvent( QDropEvent *event )
         else
         {
             bool canDrop = true;
+            bool showMessage = false;
             if( destination->type == ReplayGainFileListItem::Album )
             {
                 QList<QTreeWidgetItem*> q_items = selectedItems();
-                foreach( QTreeWidgetItem *q_item, q_items )
+                foreach( const QTreeWidgetItem *q_item, q_items )
                 {
-                    ReplayGainFileListItem *item = (ReplayGainFileListItem*)q_item;
+                    const ReplayGainFileListItem *item = static_cast<const ReplayGainFileListItem*>(q_item);
 
                     if( item->codecName != destination->codecName || item->samplingRate != destination->samplingRate )
                     {
                         canDrop = false;
+                        showMessage = true;
                         break;
                     }
                 }
             }
+            else if( destination->type == ReplayGainFileListItem::Track )
+            {
+                canDrop = false;
+            }
+
             if( canDrop )
             {
                 QTreeWidget::dropEvent(event);
             }
-            else
+            else if( showMessage )
             {
                 KMessageBox::error( this, i18n("Some tracks can't be added to the album because either the codec or the sampling rate is different.") );
             }
@@ -163,7 +170,7 @@ void ReplayGainFileList::dropEvent( QDropEvent *event )
         const bool canDecodeAlac = config->pluginLoader()->canDecode( "m4a/alac" );
         const bool checkM4a = ( !canDecodeAac || !canDecodeAlac ) && canDecodeAac != canDecodeAlac;
 
-        foreach( QUrl url, q_urls )
+        foreach( const QUrl& url, q_urls )
         {
             QString mimeType;
             QString codecName = config->pluginLoader()->getCodecFromFile( url, &mimeType, checkM4a );
@@ -266,7 +273,7 @@ void ReplayGainFileList::dropEvent( QDropEvent *event )
         {
             addFiles( k_urls );
         }
-        foreach( KUrl url, k_urls_dirs )
+        foreach( const KUrl& url, k_urls_dirs )
         {
             addDir( url, true, config->pluginLoader()->formatList(PluginLoader::ReplayGain,PluginLoader::CompressionType(PluginLoader::InferiorQuality|PluginLoader::Lossy|PluginLoader::Lossless|PluginLoader::Hybrid)) );
         }
@@ -288,18 +295,18 @@ void ReplayGainFileList::resizeEvent( QResizeEvent *event )
 int ReplayGainFileList::countDir( const QString& directory, bool recursive, int count )
 {
     QDir dir( directory );
-    dir.setFilter( QDir::Files | QDir::NoDotAndDotDot | QDir::NoSymLinks | QDir::Readable );
+    dir.setFilter( QDir::Files | QDir::NoDotAndDotDot | QDir::Readable );
     dir.setSorting( QDir::Unsorted );
 
-    count += dir.count();
+    count += (int)dir.count();
 
     if( recursive )
     {
-        dir.setFilter( QDir::Dirs | QDir::NoDotAndDotDot | QDir::NoSymLinks | QDir::Readable );
+        dir.setFilter( QDir::Dirs | QDir::NoDotAndDotDot | QDir::Readable );
 
         const QStringList list = dir.entryList();
 
-        foreach( const QString fileName, list )
+        foreach( const QString& fileName, list )
         {
             count = countDir( directory + "/" + fileName, recursive, count );
         }
@@ -320,7 +327,7 @@ int ReplayGainFileList::listDir( const QString& directory, const QStringList& fi
     QString codecName;
 
     QDir dir( directory );
-    dir.setFilter( QDir::Files | QDir::Dirs | QDir::NoDotAndDotDot | QDir::NoSymLinks | QDir::Readable );
+    dir.setFilter( QDir::Files | QDir::Dirs | QDir::NoDotAndDotDot | QDir::Readable );
     dir.setSorting( QDir::LocaleAware );
 
     const QStringList list = dir.entryList();
@@ -329,7 +336,7 @@ int ReplayGainFileList::listDir( const QString& directory, const QStringList& fi
     const bool containsAlac = filter.contains("m4a/alac");
     const bool checkM4a = ( containsAac || containsAlac ) && containsAac != containsAlac;
 
-    foreach( const QString fileName, list )
+    foreach( const QString& fileName, list )
     {
         QFileInfo fileInfo( directory + "/" + fileName );
 
@@ -365,7 +372,7 @@ void ReplayGainFileList::addFiles( const KUrl::List& fileList, const QString& _c
     ReplayGainFileListItem *newAlbumItem, *newTrackItem;
     QString codecName;
 
-    foreach( KUrl url, fileList )
+    foreach( const KUrl& url, fileList )
     {
         if( !_codecName.isEmpty() )
         {
@@ -541,24 +548,24 @@ void ReplayGainFileList::updateItem( ReplayGainFileListItem *item, bool initialU
 void ReplayGainFileList::startProcessing( ReplayGainPlugin::ApplyMode _mode )
 {
     // iterate through all items and set the state to "Waiting"
-    ReplayGainFileListItem *item, *child;
     for( int i=0; i<topLevelItemCount(); i++ )
     {
-        item = topLevelItem( i );
+        ReplayGainFileListItem *item = topLevelItem( i );
         if( item->type == ReplayGainFileListItem::Track )
         {
             bool isStopped = false;
-            if( item )
+            switch( item->state )
             {
-                switch( item->state )
+                case ReplayGainFileListItem::Waiting:
+                case ReplayGainFileListItem::WaitingForReplayGain:
+                case ReplayGainFileListItem::Processing:
                 {
-                    case ReplayGainFileListItem::Waiting:
-                        break;
-                    case ReplayGainFileListItem::Processing:
-                        break;
-                    case ReplayGainFileListItem::Stopped:
-                        isStopped = true;
-                        break;
+                    break;
+                }
+                case ReplayGainFileListItem::Stopped:
+                {
+                    isStopped = true;
+                    break;
                 }
             }
             if( isStopped )
@@ -571,39 +578,44 @@ void ReplayGainFileList::startProcessing( ReplayGainPlugin::ApplyMode _mode )
         {
             for( int j=0; j<item->childCount(); j++ )
             {
-                child = (ReplayGainFileListItem*)item->child(j);
-                bool isStopped = false;
+                ReplayGainFileListItem *child = static_cast<ReplayGainFileListItem*>(item->child(j));
                 if( child )
                 {
+                    bool isStopped = false;
                     switch( child->state )
                     {
                         case ReplayGainFileListItem::Waiting:
-                            break;
+                        case ReplayGainFileListItem::WaitingForReplayGain:
                         case ReplayGainFileListItem::Processing:
+                        {
                             break;
+                        }
                         case ReplayGainFileListItem::Stopped:
+                        {
                             isStopped = true;
                             break;
+                        }
+                    }
+                    if( isStopped )
+                    {
+                        child->state = ReplayGainFileListItem::Waiting;
+                        updateItem( child );
                     }
-                }
-                if( isStopped )
-                {
-                    child->state = ReplayGainFileListItem::Waiting;
-                    updateItem( child );
                 }
             }
             bool isStopped = false;
-            if( item )
+            switch( item->state )
             {
-                switch( item->state )
+                case ReplayGainFileListItem::Waiting:
+                case ReplayGainFileListItem::WaitingForReplayGain:
+                case ReplayGainFileListItem::Processing:
                 {
-                    case ReplayGainFileListItem::Waiting:
-                        break;
-                    case ReplayGainFileListItem::Processing:
-                        break;
-                    case ReplayGainFileListItem::Stopped:
-                        isStopped = true;
-                        break;
+                    break;
+                }
+                case ReplayGainFileListItem::Stopped:
+                {
+                    isStopped = true;
+                    break;
                 }
             }
             if( isStopped )
@@ -629,24 +641,24 @@ void ReplayGainFileList::cancelProcess()
     queue = false;
 //     emit queueModeChanged( queue );
 
-    ReplayGainFileListItem *item, *child;
     for( int i=0; i<topLevelItemCount(); i++ )
     {
-        item = topLevelItem( i );
+        ReplayGainFileListItem *item = topLevelItem( i );
         if( item->type == ReplayGainFileListItem::Track )
         {
             bool canKill = false;
-            if( item )
+            switch( item->state )
             {
-                switch( item->state )
+                case ReplayGainFileListItem::Waiting:
+                case ReplayGainFileListItem::Stopped:
                 {
-                    case ReplayGainFileListItem::Waiting:
-                        break;
-                    case ReplayGainFileListItem::Processing:
-                        canKill = true;
-                        break;
-                    case ReplayGainFileListItem::Stopped:
-                        break;
+                    break;
+                }
+                case ReplayGainFileListItem::WaitingForReplayGain:
+                case ReplayGainFileListItem::Processing:
+                {
+                    canKill = true;
+                    break;
                 }
             }
             if( canKill )
@@ -655,17 +667,18 @@ void ReplayGainFileList::cancelProcess()
         else
         {
             bool canKill = false;
-            if( item )
+            switch( item->state )
             {
-                switch( item->state )
+                case ReplayGainFileListItem::Waiting:
+                case ReplayGainFileListItem::Stopped:
                 {
-                    case ReplayGainFileListItem::Waiting:
-                        break;
-                    case ReplayGainFileListItem::Processing:
-                        canKill = true;
-                        break;
-                    case ReplayGainFileListItem::Stopped:
-                        break;
+                    break;
+                }
+                case ReplayGainFileListItem::WaitingForReplayGain:
+                case ReplayGainFileListItem::Processing:
+                {
+                    canKill = true;
+                    break;
                 }
             }
             if( canKill )
@@ -676,23 +689,27 @@ void ReplayGainFileList::cancelProcess()
             {
                 for( int j=0; j<item->childCount(); j++ )
                 {
-                    child = (ReplayGainFileListItem*)item->child(j);
-                    bool canKill = false;
+                    ReplayGainFileListItem *child = static_cast<ReplayGainFileListItem*>(item->child(j));
                     if( child )
                     {
+                        bool canKill = false;
                         switch( child->state )
                         {
                             case ReplayGainFileListItem::Waiting:
+                            case ReplayGainFileListItem::Stopped:
+                            {
                                 break;
+                            }
+                            case ReplayGainFileListItem::WaitingForReplayGain:
                             case ReplayGainFileListItem::Processing:
+                            {
                                 canKill = true;
                                 break;
-                            case ReplayGainFileListItem::Stopped:
-                                break;
+                            }
                         }
+                        if( canKill )
+                            emit killItem( child );
                     }
-                    if( canKill )
-                        emit killItem( child );
                 }
             }
         }
@@ -729,7 +746,7 @@ void ReplayGainFileList::processNextItem()
             float albumGain = 0;
             for( int j=0; j<item->childCount(); j++ )
             {
-                ReplayGainFileListItem *child = (ReplayGainFileListItem*)item->child(j);
+                ReplayGainFileListItem *child = static_cast<ReplayGainFileListItem*>(item->child(j));
                 if( child->state != ReplayGainFileListItem::Waiting )
                 {
                     childProcessing = true;
@@ -738,7 +755,7 @@ void ReplayGainFileList::processNextItem()
 
                 if( j == 0 )
                 {
-                    if( child->tags )
+                    if( child->tags && child->tags->tagsRead & TagData::AlbumGain )
                     {
                         albumGain = child->tags->albumGain;
                     }
@@ -765,7 +782,7 @@ void ReplayGainFileList::processNextItem()
             {
                 for( int j=0; j<item->childCount(); j++ )
                 {
-                    ReplayGainFileListItem *child = (ReplayGainFileListItem*)item->child(j);
+                    ReplayGainFileListItem *child = static_cast<ReplayGainFileListItem*>(item->child(j));
                     child->state = ReplayGainFileListItem::Processing;
                     updateItem( child );
                 }
@@ -782,7 +799,7 @@ void ReplayGainFileList::processNextItem()
             {
                 for( int j=0; j<item->childCount(); j++ )
                 {
-                    ReplayGainFileListItem *child = (ReplayGainFileListItem*)item->child(j);
+                    ReplayGainFileListItem *child = static_cast<ReplayGainFileListItem*>(item->child(j));
                     if( child->state == ReplayGainFileListItem::Waiting )
                     {
                         child->state = ReplayGainFileListItem::Stopped;
@@ -802,11 +819,10 @@ void ReplayGainFileList::processNextItem()
 int ReplayGainFileList::waitingCount()
 {
     int count = 0;
-    ReplayGainFileListItem *item;
 
     for( int i=0; i<topLevelItemCount(); i++ )
     {
-        item = topLevelItem( i );
+        ReplayGainFileListItem *item = topLevelItem( i );
         if( item->state == ReplayGainFileListItem::Waiting )
             count++;
     }
@@ -816,20 +832,19 @@ int ReplayGainFileList::waitingCount()
 
 int ReplayGainFileList::processingCount() // TODO use ReplayGainProcessor
 {
-    ReplayGainFileListItem *item, *child;
     int count = 0;
 
     for( int i=0; i<topLevelItemCount(); i++ )
     {
-        item = topLevelItem(i);
+        ReplayGainFileListItem *item = topLevelItem(i);
         if( item->type == ReplayGainFileListItem::Track )
         {
-            if( item->state == ReplayGainFileListItem::Processing )
+            if( item->state == ReplayGainFileListItem::Processing || item->state == ReplayGainFileListItem::WaitingForReplayGain )
                 count++;
         }
         else if( item->type == ReplayGainFileListItem::Album )
         {
-            if( item->state == ReplayGainFileListItem::Processing )
+            if( item->state == ReplayGainFileListItem::Processing || item->state == ReplayGainFileListItem::WaitingForReplayGain )
             {
                 count++;
             }
@@ -837,8 +852,8 @@ int ReplayGainFileList::processingCount() // TODO use ReplayGainProcessor
             {
                 for( int j=0; j<item->childCount(); j++ )
                 {
-                    child = (ReplayGainFileListItem*)item->child(j);
-                    if( child->state == ReplayGainFileListItem::Processing )
+                    ReplayGainFileListItem *child = static_cast<ReplayGainFileListItem*>(item->child(j));
+                    if( child->state == ReplayGainFileListItem::Processing || item->state == ReplayGainFileListItem::WaitingForReplayGain )
                     {
                         count++;
                         break;
@@ -864,7 +879,7 @@ void ReplayGainFileList::itemFinished( ReplayGainFileListItem *item, ReplayGainF
         {
             for( int j=0; j<item->childCount(); j++ )
             {
-                ReplayGainFileListItem *child = (ReplayGainFileListItem*)item->child(j);
+                ReplayGainFileListItem *child = static_cast<ReplayGainFileListItem*>(item->child(j));
                 child->state = ReplayGainFileListItem::Stopped;
                 child->returnCode = returnCode;
                 child->tags = config->tagEngine()->readTags( child->url );
@@ -899,57 +914,72 @@ void ReplayGainFileList::showContextMenu( const QPoint& point )
 {
     QList<QTreeWidgetItem*> q_items = selectedItems();
     bool canRemove = q_items.count() > 0;
-    bool canStart = q_items.count() > 0;
+//     bool canStart = q_items.count() > 0;
     bool canMove = q_items.count() > 0;
-    bool canKill = q_items.count() > 0;
+//     bool canKill = q_items.count() > 0;
 
-    foreach( QTreeWidgetItem *q_item, q_items )
+    foreach( const QTreeWidgetItem *q_item, q_items )
     {
-        ReplayGainFileListItem *item = (ReplayGainFileListItem*)q_item;
+        const ReplayGainFileListItem *item = static_cast<const ReplayGainFileListItem*>(q_item);
 
         if( item->type == ReplayGainFileListItem::Track )
         {
             switch( item->state )
             {
                 case ReplayGainFileListItem::Waiting:
-                    canKill = false;
+                {
+//                     canKill = false;
                     break;
+                }
+                case ReplayGainFileListItem::WaitingForReplayGain:
                 case ReplayGainFileListItem::Processing:
+                {
                     canRemove = false;
-                    canStart = false;
+//                     canStart = false;
                     canMove = false;
                     break;
+                }
                 case ReplayGainFileListItem::Stopped:
-                    canKill = false;
+                {
+//                     canKill = false;
                     break;
+                }
             }
         }
         else
         {
             canMove = false;
-            if( item->state == ReplayGainFileListItem::Processing )
+            if( item->state == ReplayGainFileListItem::Processing || item->state == ReplayGainFileListItem::WaitingForReplayGain )
             {
                 canRemove = false;
-                canStart = false;
+//                 canStart = false;
             }
             else
             {
                 for( int j=0; j<item->childCount(); j++ )
                 {
-                    ReplayGainFileListItem *child = (ReplayGainFileListItem*)item->child(j);
+                    ReplayGainFileListItem *child = static_cast<ReplayGainFileListItem*>(item->child(j));
 
                     switch( child->state )
                     {
                         case ReplayGainFileListItem::Waiting:
-                            canKill = false;
+                        {
+//                             canKill = false;
                             break;
+                        }
+                        case ReplayGainFileListItem::WaitingForReplayGain:
                         case ReplayGainFileListItem::Processing:
+                        {
                             canRemove = false;
-                            canStart = false;
+//                             canStart = false;
+                            canMove = false;
                             break;
+                        }
                         case ReplayGainFileListItem::Stopped:
-                            canKill = false;
+                        {
+//                             canKill = false;
                             break;
+                        }
                     }
                 }
             }
@@ -1039,7 +1069,7 @@ void ReplayGainFileList::moveSelectedItems()
     const QList<QTreeWidgetItem*> q_items = selectedItems();
     foreach( QTreeWidgetItem *q_item, q_items )
     {
-        ReplayGainFileListItem *item = dynamic_cast<ReplayGainFileListItem*>(q_item);
+        ReplayGainFileListItem *item = static_cast<ReplayGainFileListItem*>(q_item);
         if( !item )
             continue;
 
@@ -1072,11 +1102,9 @@ void ReplayGainFileList::moveSelectedItems()
 
 void ReplayGainFileList::removeSelectedItems()
 {
-    ReplayGainFileListItem *item, *child;
-
     for( int i=0; i<topLevelItemCount(); i++ )
     {
-        item = topLevelItem(i);
+        ReplayGainFileListItem *item = topLevelItem(i);
         if( item->type == ReplayGainFileListItem::Track && item->isSelected() && item->state != ReplayGainFileListItem::Processing )
         {
             emit timeChanged( -item->length );
@@ -1087,7 +1115,7 @@ void ReplayGainFileList::removeSelectedItems()
         {
             for( int j=0; j<item->childCount(); j++ )
             {
-                child = (ReplayGainFileListItem*)item->child(j);
+                ReplayGainFileListItem *child = static_cast<ReplayGainFileListItem*>(item->child(j));
                 if( child->type == ReplayGainFileListItem::Track && ( child->isSelected() || item->isSelected() ) && child->state != ReplayGainFileListItem::Processing )
                 {
                     emit timeChanged( -child->length );
