@@ -29,6 +29,7 @@
 #include <KActionMenu>
 #include <KMessageBox>
 
+#include <QApplication>
 #include <QLabel>
 #include <QLayout>
 #include <QHBoxLayout>
@@ -45,6 +46,8 @@ soundKonverterView::soundKonverterView( Logger *_logger, Config *_config, CDMana
 {
     setAcceptDrops( true );
 
+    const int fontHeight = QFontMetrics(QApplication::font()).boundingRect("M").size().height();
+
     // the grid for all widgets in the main window
     QGridLayout* gridLayout = new QGridLayout( this );
 
@@ -67,7 +70,7 @@ soundKonverterView::soundKonverterView( Logger *_logger, Config *_config, CDMana
 
     // add a horizontal box layout for the add combobutton to the grid
     QHBoxLayout *addBox = new QHBoxLayout();
-    addBox->setContentsMargins( 1, 2, 1, 0 ); // determined by experiments
+    addBox->setContentsMargins( 1, 0, 1, 0 ); // extra margin - determined by experiments
     gridLayout->addLayout( addBox, 3, 0 );
 
     // create the combobutton for adding files to the file list
@@ -81,12 +84,12 @@ soundKonverterView::soundKonverterView( Logger *_logger, Config *_config, CDMana
     cAdd->insertItem( KIcon("media-optical-audio"), i18n("Add CD tracks...") );
     cAdd->insertItem( KIcon("network-workgroup"), i18n("Add url...") );
     cAdd->insertItem( KIcon("view-media-playlist"), i18n("Add playlist...") );
-    cAdd->increaseHeight( 6 );
+    cAdd->increaseHeight( 0.6*fontHeight );
     addBox->addWidget( cAdd, 0, Qt::AlignVCenter );
     connect( cAdd, SIGNAL(clicked(int)), this, SLOT(addClicked(int)) );
     cAdd->setFocus();
 
-    addBox->addSpacing( 10 );
+    addBox->addSpacing( fontHeight );
 
     startAction = new KAction( KIcon("system-run"), i18n("Start"), this );
     connect( startAction, SIGNAL(triggered()), fileList, SLOT(startConversion()) );
@@ -99,7 +102,8 @@ soundKonverterView::soundKonverterView( Logger *_logger, Config *_config, CDMana
     connect( pStart, SIGNAL(clicked()), fileList, SLOT(startConversion()) );
 
     stopActionMenu = new KActionMenu( KIcon("process-stop"), i18n("Stop"), this );
-    killAction = new KAction( KIcon("flag-red"), i18n("Stop imediatelly"), this );
+    stopActionMenu->setDelayed( false );
+    killAction = new KAction( KIcon("flag-red"), i18n("Stop immediatelly"), this );
     stopActionMenu->addAction( killAction );
     connect( killAction, SIGNAL(triggered()), fileList, SLOT(killConversion()) );
     stopAction = new KAction( KIcon("flag-yellow"), i18n("Stop after current conversions are completed"), this );
@@ -117,7 +121,7 @@ soundKonverterView::soundKonverterView( Logger *_logger, Config *_config, CDMana
     pStop->setMenu( stopActionMenu->menu() );
     addBox->addWidget( pStop, 0, Qt::AlignVCenter );
 
-    addBox->addSpacing( 10 );
+    addBox->addSpacing( fontHeight );
 
     progressIndicator = new ProgressIndicator( this, ProgressIndicator::Feature( ProgressIndicator::FeatureSpeed | ProgressIndicator::FeatureTime ) );
     addBox->addWidget( progressIndicator, 0, Qt::AlignVCenter );
@@ -397,15 +401,27 @@ void soundKonverterView::addConvertFiles( const KUrl::List& urls, QString _profi
 
         cleanupParameters( &profile, &format );
 
-        if( !profile.isEmpty() && !format.isEmpty() && !directory.isEmpty() )
+        const bool isUserProfile = config->data.profiles.contains(profile);
+
+        if( ( !profile.isEmpty() && !format.isEmpty() && !directory.isEmpty() ) || isUserProfile )
         {
-            Options *options = new Options( config, "", 0 );
-            options->hide();
-            options->setProfile( profile );
-            options->setFormat( format );
-            options->setOutputDirectory( directory );
-            ConversionOptions *conversionOptions = options->currentConversionOptions();
-            delete options;
+            ConversionOptions *conversionOptions = 0;
+
+            if( !isUserProfile )
+            {
+                Options *options = new Options( config, "", 0 );
+                options->hide();
+                options->setProfile( profile );
+                options->setFormat( format );
+                options->setOutputDirectory( directory );
+                conversionOptions = options->currentConversionOptions();
+                delete options;
+            }
+            else
+            {
+                conversionOptions = config->data.profiles.value( profile )->copy();
+            }
+
             if( conversionOptions )
             {
                 fileList->addFiles( k_urls, conversionOptions, notifyCommand );
@@ -414,7 +430,7 @@ void soundKonverterView::addConvertFiles( const KUrl::List& urls, QString _profi
             {
                 // FIXME error message, null pointer for conversion options
 //                 KMessageBox::error( this, i18n("Sorry, this shouldn't happen.\n\nPlease report this bug and attach the following error message:\n\nsoundKonverterView::addConvertFiles; Options::currentConversionOptions returned 0"), i18n("Internal error") );
-                KMessageBox::error( this, "Sorry, this shouldn't happen.\n\nPlease report this bug and attach the following error message:\n\nsoundKonverterView::addConvertFiles; Options::currentConversionOptions returned 0", "Internal error" );
+                KMessageBox::error( this, "Sorry, this shouldn't happen.\n\nPlease report this bug and attach the following error message:\n\nsoundKonverterView::addConvertFiles; conversionOptions=0, isUserProfile="+QString::number(isUserProfile), "Internal error" );
             }
         }
         else
@@ -445,6 +461,11 @@ void soundKonverterView::loadAutosaveFileList()
     fileList->load( false );
 }
 
+void soundKonverterView::loadFileList(const QString& fileListPath)
+{
+    fileList->load( fileListPath );
+}
+
 void soundKonverterView::startConversion()
 {
     fileList->startConversion();
@@ -515,7 +536,7 @@ void soundKonverterView::cleanupParameters( QString *profile, QString *format )
     }
     else
     {
-        foreach( const QString format, formatList )
+        foreach( const QString& format, formatList )
         {
             if( config->pluginLoader()->codecExtensions(format).contains(old_format) )
             {
@@ -563,7 +584,7 @@ void soundKonverterView::cleanupParameters( QString *profile, QString *format )
     else if( config->data.profiles.contains(old_profile) )
     {
         new_profile = old_profile;
-        ConversionOptions *conversionOptions = config->data.profiles.value( new_profile );
+        const ConversionOptions *conversionOptions = config->data.profiles.value( new_profile );
         if( conversionOptions )
             new_format += conversionOptions->codecName;
     }
